@@ -41,6 +41,7 @@
 #include "netlink.h"
 #include "odp-util.h"
 #include "openvswitch/ofpbuf.h"
+#include "openvswitch/ofp-ct.h"
 #include "packets.h"
 #include "openvswitch/shash.h"
 #include "simap.h"
@@ -672,7 +673,7 @@ show_dpif(struct dpif *dpif, struct dpctl_params *dpctl_p)
     }
 
     for (int i = 0; i < n_port_nos; i++) {
-        if (dpif_port_query_by_number(dpif, port_nos[i], &dpif_port, true)) {
+        if (dpif_port_query_by_number(dpif, port_nos[i], &dpif_port)) {
             continue;
         }
 
@@ -725,8 +726,8 @@ show_dpif(struct dpif *dpif, struct dpctl_params *dpctl_p)
                 continue;
             }
             error = netdev_get_stats(netdev, &s);
-            netdev_close(netdev);
             if (!error) {
+                netdev_close(netdev);
                 print_stat(dpctl_p, "    RX packets:", s.rx_packets);
                 print_stat(dpctl_p, " errors:", s.rx_errors);
                 print_stat(dpctl_p, " dropped:", s.rx_dropped);
@@ -1342,17 +1343,19 @@ static int
 dpctl_del_flow_dpif(struct dpif *dpif, const char *key_s,
                     struct dpctl_params *dpctl_p)
 {
-    struct dpif_port_dump port_dump;
     struct dpif_flow_stats stats;
-    bool ufid_generated = false;
     struct dpif_port dpif_port;
-    bool ufid_present = false;
-    struct simap port_names;
-    struct ofpbuf mask; /* To be ignored. */
+    struct dpif_port_dump port_dump;
     struct ofpbuf key;
+    struct ofpbuf mask; /* To be ignored. */
+
     ovs_u128 ufid;
+    bool ufid_generated;
+    bool ufid_present;
+    struct simap port_names;
     int n, error;
 
+    ufid_present = false;
     n = odp_ufid_from_string(key_s, &ufid);
     if (n < 0) {
         dpctl_error(dpctl_p, -n, "parsing flow ufid");
@@ -1705,43 +1708,64 @@ dpctl_flush_conntrack(int argc, const char *argv[],
                       struct dpctl_params *dpctl_p)
 {
     struct dpif *dpif = NULL;
-    struct ct_dpif_tuple tuple, *ptuple = NULL;
+    struct ofp_ct_match match = {0};
     struct ds ds = DS_EMPTY_INITIALIZER;
     uint16_t zone, *pzone = NULL;
     int error;
     int args = argc - 1;
 
-    /* Parse ct tuple */
-    if (args && ct_dpif_parse_tuple(&tuple, argv[args], &ds)) {
-        ptuple = &tuple;
-        args--;
-    }
-
-    /* Parse zone */
-    if (args && ovs_scan(argv[args], "zone=%"SCNu16, &zone)) {
+    /* Parse zone. */
+    if (args && !strncmp(argv[1], "zone=", 5)) {
+        if (!ovs_scan(argv[1], "zone=%"SCNu16, &zone)) {
+            ds_put_cstr(&ds, "failed to parse zone");
+            error = EINVAL;
+            goto error;
+        }
         pzone = &zone;
         args--;
     }
 
-    /* Report error if there are more than one unparsed argument. */
+    /* Parse ct tuples. */
+    for (int i = 0; i < 2; i++) {
+        if (!args) {
+            break;
+        }
+
+        struct ofp_ct_tuple *tuple =
+            i ? &match.tuple_reply : &match.tuple_orig;
+        const char *arg = argv[argc - args];
+
+        if (arg[0] && !ofp_ct_tuple_parse(tuple, arg, &ds, &match.ip_proto,
+                                          &match.l3_type)) {
+            error = EINVAL;
+            goto error;
+        }
+        args--;
+    }
+
+    /* Report error if there is more than one unparsed argument. */
     if (args > 1) {
+        ds_put_cstr(&ds, "invalid arguments");
         error = EINVAL;
-        dpctl_error(dpctl_p, error, "invalid arguments: %s", ds_cstr(&ds));
-        goto out;
+        goto error;
     }
 
-    error = opt_dpif_open(argc, argv, dpctl_p, 4, &dpif);
+    error = opt_dpif_open(argc, argv, dpctl_p, 5, &dpif);
     if (error) {
-        goto out;
+        dpctl_error(dpctl_p, error, "Cannot open dpif");
+        return error;
     }
 
-    error = ct_dpif_flush(dpif, pzone, ptuple);
-    if (error) {
-        dpctl_error(dpctl_p, error, "failed to flush conntrack: %s",
-                    ds_cstr(&ds));
+    error = ct_dpif_flush(dpif, pzone, &match);
+    if (!error) {
+        dpif_close(dpif);
+        return 0;
+    } else {
+        ds_put_cstr(&ds, "failed to flush conntrack");
     }
 
-out:
+error:
+    dpctl_error(dpctl_p, error, "%s", ds_cstr(&ds));
     ds_destroy(&ds);
     dpif_close(dpif);
     return error;
@@ -2172,7 +2196,7 @@ parse_ct_limit_zones(const char *argv, struct ovs_list *zone_limits,
     argcopy = xstrdup(argv + 5);
     next_zone = strtok_r(argcopy, ",", &save_ptr);
 
-    while (next_zone != NULL) {
+    do {
         if (ovs_scan(next_zone, "%"SCNu16, &zone)) {
             ct_dpif_push_zone_limit(zone_limits, zone, 0, 0);
         } else {
@@ -2180,8 +2204,7 @@ parse_ct_limit_zones(const char *argv, struct ovs_list *zone_limits,
             free(argcopy);
             return EINVAL;
         }
-        next_zone = strtok_r(NULL, ",", &save_ptr);
-    }
+    } while ((next_zone = strtok_r(NULL, ",", &save_ptr)) != NULL);
 
     free(argcopy);
     return 0;
@@ -2858,8 +2881,8 @@ static const struct dpctl_command all_commands[] = {
       0, 1, dpctl_offload_stats_show, DP_RO },
     { "dump-conntrack", "[-m] [-s] [dp] [zone=N]",
       0, 4, dpctl_dump_conntrack, DP_RO },
-    { "flush-conntrack", "[dp] [zone=N] [ct-tuple]", 0, 3,
-      dpctl_flush_conntrack, DP_RW },
+    { "flush-conntrack", "[dp] [zone=N] [ct-orig-tuple] [ct-reply-tuple]",
+      0, 4, dpctl_flush_conntrack, DP_RW },
     { "cache-get-size", "[dp]", 0, 1, dpctl_cache_get_size, DP_RO },
     { "cache-set-size", "dp cache <size>", 3, 3, dpctl_cache_set_size, DP_RW },
     { "ct-stats-show", "[dp] [zone=N]",
