@@ -100,7 +100,7 @@ smap_add_format(struct smap *smap, const char *key, const char *format, ...)
 /* Adds 'key' paired with a string representation of 'addr'. It is the
  * caller's responsibility to avoid duplicate keys if desirable. */
 void
-smap_add_ipv6(struct smap *smap, const char *key, const struct in6_addr *addr)
+smap_add_ipv6(struct smap *smap, const char *key, struct in6_addr *addr)
 {
     char buf[INET6_ADDRSTRLEN];
     ipv6_string_mapped(buf, addr);
@@ -310,6 +310,8 @@ smap_clone(struct smap *dst, const struct smap *src)
     const struct smap_node *node;
 
     smap_init(dst);
+    hmap_reserve(&dst->map, smap_count(src));
+
     SMAP_FOR_EACH (node, src) {
         smap_add__(dst, xstrdup(node->key), xstrdup(node->value),
                    node->node.hash);
